import json
from datetime import datetime

import urllib.parse
import requests
from omnik.plugins import Plugin

class pvoutput(Plugin):

  def __init__(self):
    super().__init__()
    self.name = 'pvoutput'
    self.description = 'Write output to PVOutput'

  def get_weather(self):
    try:
      if 'weather' not in self.cache:
        self.logger.debug('[cache miss] Fetching weather data')
        url = "https://{endpoint}/data/2.5/weather?lon={lon}&lat={lat}&units={units}&APPID={api_key}".format(
          endpoint = self.config.get('openweathermap', 'endpoint'),
          lat      = self.config.get('openweathermap', 'lat'),
          lon      = self.config.get('openweathermap', 'lon'),
          units    = self.config.get('openweathermap', 'units', fallback='metric'),
          api_key  = self.config.get('openweathermap', 'api_key'),
        )

        res = requests.get(url)

        res.raise_for_status()

        self.cache['weather'] = res.json()        

      return self.cache['weather']

    except requests.exceptions.HTTPError as e:
      self.logger.error('Unable to get data. [{0}]: {1}'.format(type(e).__name__, str(e)))
      raise e

  def process(self, **args):
    """
    Send data to pvoutput
    """
    now = datetime.now()

    msg = args['msg']

    self.logger.debug(json.dumps(msg, indent=2))

    if not self.config.has_option('pvoutput', 'sys_id') or not self.config.has_option('pvoutput', 'api_key'):
      self.logger.error('[{}] No api_key and/or sys_id found in configuration'.format(__name__))
      return

    headers = {
      "X-Pvoutput-Apikey": self.config.get('pvoutput', 'api_key'),
      "X-Pvoutput-SystemId": self.config.get('pvoutput', 'sys_id'),
      "Content-type": "application/x-www-form-urlencoded",
      "Accept": "text/plain"
    }

    # See: https://pvoutput.org/help.html
    data = {
        'd': now.strftime('%Y%m%d'),
        't': now.strftime('%H:%M'),
        'v1': str(float(msg['today_energy']) * 1000),
        'v2': str(float(msg['current_power']) * 1000)
    }

    if self.config.getboolean('pvoutput', 'use_temperature', fallback=False):
      weather = self.get_weather()

      data['v5'] = str(weather['main']['temp'])

    encoded = urllib.parse.urlencode(data)

    self.logger.debug(json.dumps(data, indent=2))

    r = requests.post("http://pvoutput.org/service/r2/addstatus.jsp", data=encoded, headers=headers)
    
    r.raise_for_status()



    

    


import click

from dsmr import __version__
from dsmr.dsmr import DSMRReader

@click.group()
@click.version_option(version=__version__)
def main():
    pass

@main.command()
def log():
    click.echo('log')

@main.command()
@click.option('-d', '--debug', is_flag=True)
def read(debug):
    clazz = DSMRReader(debug=debug)

    clazz.read()


import os
import sys
import logging
from datetime import datetime

logger = logging.getLogger(__name__)

class DataLogger(object):

  def __init__(self, config, debug=False):
    self.config = config
    self.plugins = []

    if debug:
      logger.setLevel(logging.DEBUG)



from cachetools import TTLCache

# TODO: Create Abstract Base Class
class BasePlugin(type):
  def __init__(cls, name, bases, attrs):
    super(BasePlugin, cls).__init__(name)
    if not hasattr(cls, 'plugins'):
      cls.plugins = []
    else:
      cls.register(cls) # Called when a plugin class is imported

  def register(cls, plugin):
    cls.plugins.append(plugin())

class Plugin(object, metaclass=BasePlugin):
  
  config = None
  logger = None

  cache  = TTLCache(maxsize=1, ttl=300)


import os
import time
import json
from dsmr_parser import telegram_specifications
from dsmr_parser.clients import SerialReader, SERIAL_SETTINGS_V2_2
from dsmr_parser import obis_references as obis_ref

class DSMREntity(object):
  """
  Entity reading values from DSMR telegram.
  """

  def __init__(self, name, obis):
    """Initialize entity."""
    self._name = name
    self._obis = obis
    self.telegram = {}

  def get_dsmr_object_attr(self, attribute):
    """Read attribute from last received telegram for this DSMR object."""
    # Make sure telegram contains an object for this entities obis
    if self._obis not in self.telegram:
        return None

    # Get the attribute value if the object has it
    dsmr_object = self.telegram[self._obis]
    return getattr(dsmr_object, attribute, None)

  @property
  def name(self):
    """Return the name of the sensor."""
    return self._name

  def state(self):
    """Return the state of sensor, if available, translate if needed."""
    from dsmr_parser import obis_references as obis

    value = self.get_dsmr_object_attr('value')

    if self._obis == obis.ELECTRICITY_ACTIVE_TARIFF:
      return self.translate_tariff(value)

    try:
      value = round(float(value), 2)

    except TypeError:
      pass

    return value

  @property
  def unit_of_measurement(self):
    """Return the unit of measurement of this entity, if any."""
    return self.get_dsmr_object_attr('unit')

  @staticmethod
  def translate_tariff(value):
    """Convert 2/1 to normal/low."""
    # DSMR V2.2: Note: Rate code 1 is used for low rate and rate code 2 is
    # used for normal rate.
    if value == '0002':
      return 'normal'
    elif value == '0001':
      return 'low'
    else:
      return None

class DSMRReader():

    def __init__(self, device = '/dev/ttyUSB0', serial_settings = SERIAL_SETTINGS_V2_2, telegram_specification = telegram_specifications.V2_2, debug=False):
        self.device                 = device
        self.serial_settings        = serial_settings
        self.telegram_specification = telegram_specification
        self.debug                  = debug

        self.serial_reader = SerialReader(
            device                 = self.device,
            serial_settings        = self.serial_settings,
            telegram_specification = self.telegram_specification
        )

        self.obis_mapping_json = [
            [
                'power_consumption',
                obis_ref.CURRENT_ELECTRICITY_USAGE
            ],
            [
                'power_production',
                obis_ref.CURRENT_ELECTRICITY_DELIVERY
            ],
            [
                'power_tariff',
                obis_ref.ELECTRICITY_ACTIVE_TARIFF
            ],
            [
                'power_consumption_low',
                obis_ref.ELECTRICITY_USED_TARIFF_1
            ],
            [
                'power_consumption_normal',
                obis_ref.ELECTRICITY_USED_TARIFF_2
            ],
            [
                'power_production_low',
                obis_ref.ELECTRICITY_DELIVERED_TARIFF_1
            ],
            [
                'power_production_normal',
                obis_ref.ELECTRICITY_DELIVERED_TARIFF_2
            ],
            [
                'gas_consumption',
                obis_ref.GAS_METER_READING
            ]
        ]

        self.obis_mapping = [
            [
                'Power Consumption',
                obis_ref.CURRENT_ELECTRICITY_USAGE
            ],
            [
                'Power Production',
                obis_ref.CURRENT_ELECTRICITY_DELIVERY
            ],
            [
                'Power Tariff',
                obis_ref.ELECTRICITY_ACTIVE_TARIFF
            ],
            [
                'Power Consumption (low)',
                obis_ref.ELECTRICITY_USED_TARIFF_1
            ],
            [
                'Power Consumption (normal)',
                obis_ref.ELECTRICITY_USED_TARIFF_2
            ],
            [
                'Power Production (low)',
                obis_ref.ELECTRICITY_DELIVERED_TARIFF_1
            ],
            [
                'Power Production (normal)',
                obis_ref.ELECTRICITY_DELIVERED_TARIFF_2
            ]
            # ,
            # [
            #     'Power Consumption Phase L1',
            #     obis_ref.INSTANTANEOUS_ACTIVE_POWER_L1_POSITIVE
            # ],
            # [
            #     'Power Consumption Phase L2',
            #     obis_ref.INSTANTANEOUS_ACTIVE_POWER_L2_POSITIVE
            # ],
            # [
            #     'Power Consumption Phase L3',
            #     obis_ref.INSTANTANEOUS_ACTIVE_POWER_L3_POSITIVE
            # ],
            # [
            #     'Power Production Phase L1',
            #     obis_ref.INSTANTANEOUS_ACTIVE_POWER_L1_NEGATIVE
            # ],
            # [
            #     'Power Production Phase L2',
            #     obis_ref.INSTANTANEOUS_ACTIVE_POWER_L2_NEGATIVE
            # ],
            # [
            #     'Power Production Phase L3',
            #     obis_ref.INSTANTANEOUS_ACTIVE_POWER_L3_NEGATIVE
            # ],
            # [
            #     'Long Power Failure Count',
            #     obis_ref.LONG_POWER_FAILURE_COUNT
            # ],
            # [
            #     'Voltage Sags Phase L1',
            #     obis_ref.VOLTAGE_SAG_L1_COUNT
            # ],
            # [
            #     'Voltage Sags Phase L2',
            #     obis_ref.VOLTAGE_SAG_L2_COUNT
            # ],
            # [
            #     'Voltage Sags Phase L3',
            #     obis_ref.VOLTAGE_SAG_L3_COUNT
            # ],
            # [
            #     'Voltage Swells Phase L1',
            #     obis_ref.VOLTAGE_SWELL_L1_COUNT
            # ],
            # [
            #     'Voltage Swells Phase L2',
            #     obis_ref.VOLTAGE_SWELL_L2_COUNT
            # ],
            # [
            #     'Voltage Swells Phase L3',
            #     obis_ref.VOLTAGE_SWELL_L3_COUNT
            # ],
            # [
            #     'Voltage Phase L1',
            #     obis_ref.INSTANTANEOUS_VOLTAGE_L1
            # ],
            # [
            #     'Voltage Phase L2',
            #     obis_ref.INSTANTANEOUS_VOLTAGE_L2
            # ],
            # [
            #     'Voltage Phase L3',
            #     obis_ref.INSTANTANEOUS_VOLTAGE_L3
            # ],
            ,
            [
              'Gas Consumption',
              obis_ref.GAS_METER_READING
            ]
        ]

        # self.devices = [DSMREntity(name, obis) for name, obis in self.obis_mapping]
        self.devices = [DSMREntity(name, obis) for name, obis in self.obis_mapping_json]

    def extract_data_from_telegram(self, telegram):
      '''
      Transform telegram to json
      '''
      # solar = self.read_solar()

      data = dict(
        unix_timestamp = int(time.time()),
        current_electricity_usage = str(telegram[obis_ref.CURRENT_ELECTRICITY_USAGE].value * 1000),
        current_electricity_delivery = str(telegram[obis_ref.CURRENT_ELECTRICITY_DELIVERY].value * 1000),
        electricity_active_tariff = str(telegram[obis_ref.ELECTRICITY_ACTIVE_TARIFF].value),
        usage_total_low = str(telegram[obis_ref.ELECTRICITY_USED_TARIFF_1].value * 1000),
        usage_total_high = str(telegram[obis_ref.ELECTRICITY_USED_TARIFF_2].value * 1000),
        redelivery_total_low = str(telegram[obis_ref.ELECTRICITY_DELIVERED_TARIFF_1].value * 1000),
        redelivery_total_high = str(telegram[obis_ref.ELECTRICITY_DELIVERED_TARIFF_2].value * 1000),
        usage_gas_total = str(telegram[obis_ref.HOURLY_GAS_METER_READING].value * 1000)
      )

      if self.debug:
          print(json.dumps(data, indent=2))

      return data

    def __update_entities_telegram(self, telegram):
        '''
        Update entities with latest telegram and trigger state update.
        '''
        # Make all device entities aware of new telegram
        for device in self.devices:
            device.telegram = telegram
        

    def read(self):
        '''
        Read DSMR telegrams
        '''
        try:
          if self.debug:
            print('>> Reading DSMR in debug mode') 

          t = dict()

          for telegram in self.serial_reader.read_as_object():
            # now, translate telegram to dict
            obj = self.extract_data_from_telegram(telegram)

            # do something with 'obj'
            

          # for telegram in self.serial_reader.read():

          #   if (self.debug):
          #     print(telegram)

          #   self.__update_entities_telegram(telegram)

          #   data = dict()

          #   for device in self.devices:
          #       if device.unit_of_measurement == 'kW':
          #           value = float(device.state()) * 1000
          #       else:
          #           value = device.state()

          #       # print(f'{device.name} == {value} {device.unit_of_measurement or ""}')

          #       data[device.name] = value

          #   print(json.dumps(data, indent=2))


          #   print('=' * 80)

        except Exception as e:
          print(str(e))
        
        



import os
import sys
import toml
from pathlib import Path
import logging
import time
from threading import Timer

from .datalogger import DataLogger

logging.basicConfig(stream=sys.stdout, level=os.environ.get('LOGLEVEL', logging.INFO))

def get_version():
   path = Path(__file__).resolve().parents[1] / 'pyproject.toml'
   pyproject = toml.loads(open(str(path)).read())
   return pyproject['tool']['poetry']['version']

__version__ = get_version()

class RepeatedJob(object):
  def __init__(self, interval, function, *args, **kwargs):
    self._timer     = None
    self.interval   = interval
    self.function   = function  
    self.args       = args
    self.kwargs     = kwargs
    self.is_running = False
    self.start()

  def _run(self):
    self.is_running = False
    self.start()
    self.function(*self.args, **self.kwargs)

  def start(self):
    if not self.is_running:
      self._timer = Timer(self.interval, self._run)
      self._timer.daemon = True
      self._timer.start()
      self.is_running = True

  def stop(self):
    self._timer.cancel()
    self.is_running = False


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['dsmr', 'dsmr.dsmr', 'dsmr.plugins']

package_data = \
{'': ['*']}

install_requires = \
['click>=7.1.2,<8.0.0',
 'configparser>=5.0.0,<6.0.0',
 'dsmr_parser>=0.18,<0.19',
 'toml>=0.10.0,<0.11.0']

entry_points = \
{'console_scripts': ['dsmr = dsmr.cli:main']}

setup_kwargs = {
    'name': 'dsmr-data-logger',
    'version': '0.1.7',
    'description': 'DSMR data logger',
    'long_description': '# p1-data-logger',
    'author': 'Pascal Prins',
    'author_email': 'pascal.prins@foobar-it.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/paprins/dsmr',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
